#include <stdio.h>
#include <string.h>
#include "cpp.h"

#define ERR(str, chr) if (opterr) { fprintf(ERROUT, "%s%c\n", str, chr); }

int    opterr = 1;
int    optind = 1;
int    optopt;
char * optarg;


/************************************************************************/
/* -- The parameter opts is a string which contains the possible option */
/* -- characters. If the option character is followed by ':', a         */
/* -- parameter is also read from the command line and the pointer      */
/* -- optarg points to this parameter. The variable optopt contains the */
/* -- option character on return.                                       */
/************************************************************************/

int getopt(int argc, char **argv, char *opts)
{
    static int sp = 1;
    int        c;
    char *     cp;

    if (sp == 1)
    {
        if ((optind >= argc)
        || (argv[optind][0] != '-')
        || (argv[optind][1] == '\0'))
        {
            return -1;
        }

        else if (strcmp(argv[optind], "--") == 0)
        {
            optind++;
            return -1;
        }
    }

    optopt = c = argv[optind][sp];
    if ((c == ':') || ((cp = strchr(opts, c)) == 0))
    {
        ERR(": illegal option -- ", c);
        if (argv[optind][++sp] == '\0')
        {
            optind++;
            sp = 1;
        }
        return '?';
    }

    if (*++cp == ':')
    {
        /* -- This option takes a parameter       */

        if (argv[optind][sp + 1] != '\0')
        {
            optarg = &argv[optind++][sp + 1];
        }

        else if (++optind >= argc)
        {
            ERR(": option requires an argument -- ", c);
            sp = 1;
            return '?';
        }

        else
        {
            optarg = argv[optind++];
        }

        sp = 1;
    }
    else
    {
        if (argv[optind][++sp] == '\0')
        {
            sp = 1;
            optind++;
        }

        optarg = NULL;
    }

    return c;
}
